<?php

namespace Drupal\yasm\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\yasm\Utility\EntitiesStatistics;
use Drupal\yasm\Utility\YasmHtml;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * YASM Statistics dashboard controller.
 */
class Dashboard extends ControllerBase {

  /**
   * The current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Page site content output.
   */
  public function siteContent() {
    return $this->buildContent();
  }

  /**
   * Page my content output.
   */
  public function myContent() {
    return $this->buildContent(['uid' => $this->currentUser->id()]);
  }

  /**
   * Build content output.
   */
  private function buildContent(array $conditions = []) {

    if (isset($conditions['uid'])) {
      $this->messenger->addMessage($this->t('Statistics filtered with content authored by you.'));
      $site_stats = FALSE;
    }
    else {
      $this->messenger->addMessage($this->t('Expensive statistics updates their data 1 time an hour, less expensive statistics updates when the data changes.'));
      $site_stats = TRUE;
    }

    $cards = [];
    // User and site stats.
    if ($this->moduleHandler->moduleExists('file')) {
      $cards[] = YasmHtml::picto('far fa-file') . $this->t('Files') . ' : ' . EntitiesStatistics::count('file', $conditions);
    }
    if ($this->moduleHandler->moduleExists('node')) {
      $cards[] = YasmHtml::picto('far fa-file-alt') . $this->t('Nodes') . ': ' . EntitiesStatistics::count('node', $conditions);
    }
    if ($this->moduleHandler->moduleExists('comment')) {
      $cards[] = YasmHtml::picto('fas fa-comment') . $this->t('Comments') . ': ' . EntitiesStatistics::count('comment', $conditions);
    }

    // Only site stats.
    if ($site_stats) {
      if ($this->moduleHandler->moduleExists('user')) {
        $cards[] = YasmHtml::picto('fas fa-user') . $this->t('Users') . ': ' . EntitiesStatistics::count('user');
      }
      if ($this->moduleHandler->moduleExists('taxonomy')) {
        $cards[] = YasmHtml::picto('fas fa-tags') . $this->t('Terms') . ': ' . EntitiesStatistics::count('taxonomy_term');
      }
      if ($this->moduleHandler->moduleExists('group')) {
        $cards[] = YasmHtml::picto('fas fa-users') . $this->t('Groups') . ': ' . EntitiesStatistics::count('group');
      }
      if ($this->moduleHandler->moduleExists('menu_link_content')) {
        $cards[] = YasmHtml::picto('fas fa-bars') . $this->t('Menus') . ': ' . EntitiesStatistics::count('menu');
      }
      if ($this->moduleHandler->moduleExists('block')) {
        $cards[] = YasmHtml::picto('fas fa-th-large') . $this->t('Blocks') . ': ' . EntitiesStatistics::count('block');
      }
      if ($this->moduleHandler->moduleExists('views')) {
        $cards[] = YasmHtml::picto('far fa-list-alt') . $this->t('Views') . ': ' . EntitiesStatistics::count('view');
      }
    }

    $build = [];
    $build[] = YasmHtml::columns($cards, ['yasm-dashboard']);
    $build[] = [
      '#attached' => [
        'library' => ['yasm/global', 'yasm/fontawesome'],
      ],
      '#cache' => ['max-age' => 3600],
    ];

    return $build;
  }

  /**
   * Constructs the controller object.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The user from current session.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   */
  public function __construct(AccountInterface $current_user, MessengerInterface $messenger, ModuleHandlerInterface $module_handler) {
    $this->currentUser = $current_user;
    $this->messenger = $messenger;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('messenger'),
      $container->get('module_handler')
    );
  }

}
