<?php

namespace Drupal\yasm\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\group\GroupMembership;
use Drupal\group\Entity\Group;
use Drupal\yasm\Utility\GroupsStatistics;
use Drupal\yasm\Utility\YasmHtml;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * YASM Statistics site groups controller.
 */
class Groups extends ControllerBase {

  /**
   * The current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * A custom access check.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result.
   */
  public function access(AccountInterface $account) {
    return ($this->moduleHandler->moduleExists('group')) ? AccessResult::allowed() : AccessResult::forbidden();
  }

  /**
   * Site groups page output.
   */
  public function siteContent() {
    // Get all groups.
    $site_groups = $this->entityTypeManager->getStorage('group')->loadMultiple();
    $groups = $this->getGroupsContents($site_groups);

    // Get all content types.
    $content_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();

    // Create groups table.
    $rows = [];
    foreach ($groups as $group) {
      $row = [
        'type'  => $group['type'],
        'name'  => $group['name'],
        'members'  => $group['members'],
        'contents' => $group['contents'],
      ];
      foreach ($content_types as $key => $value) {
        $row[$key] = isset($group['by_type'][$key]) ? $group['by_type'][$key]['count'] : 0;
      }

      $rows[] = $row;
    }

    $labels = [
      $this->t('Type'),
      $this->t('Group'),
      $this->t('Members'),
      $this->t('Total contents'),
    ];
    foreach ($content_types as $content_type) {
      $labels[] = $content_type->label();
    }

    // Build array content output.
    $build = [];
    $build[] = YasmHtml::title($this->t('Groups'), 'fas fa-users');
    $build[] = YasmHtml::table($labels, $rows);

    return $this->buildContents($build);
  }

  /**
   * My groups page output.
   */
  public function myContent() {
    $user_groups = \Drupal::service('group.membership_loader')->loadByUser($this->currentUser);
    $groups = $this->getGroupsContents($user_groups);

    $this->messenger->addMessage($this->t('Statistics filtered with your groups membership.'));

    // Build content output.
    $cards = [];
    foreach ($groups as $group) {
      $card = [];
      $card[] = YasmHtml::title($group['name'] . ' (' . $group['type'] . ')', 'fas fa-users', ['panel-title']);
      $card[] = YasmHtml::markup($this->t('Members') . ': ' . $group['members'], 'fas fa-user', ['yasm-group-datakey']);
      $card[] = YasmHtml::markup($this->t('Contents') . ': ' . $group['contents'], 'far fa-file-alt', ['yasm-group-datakey']);

      if (!empty($group['by_type'])) {
        $card[] = YasmHtml::table([$this->t('Type'), $this->t('Count')], $group['by_type']);
      }

      $cards[] = $card;
    }

    $build = YasmHtml::columns($cards, ['yasm-groups'], 2);

    return $this->buildContents($build);
  }

  /**
   * Build output attaching libraris and cache settings.
   */
  private function buildContents(array $build) {
    $build[] = [
      '#attached' => [
        'library' => ['yasm/global', 'yasm/fontawesome', 'yasm/datatables'],
      ],
      '#cache' => [
        '#cache' => ['max-age' => 3600],
      ],
    ];

    return $build;
  }

  /**
   * Get groups statistics.
   */
  private function getGroupsContents(array $groups) {
    $groups_stats = [];
    foreach ($groups as $group) {
      $g = ($group instanceof GroupMembership) ? $group->getGroup() : $group;
      if ($g instanceof Group) {
        $groups_stats[$g->id()] = [
          'name'     => $g->label(),
          'type'     => $g->getGroupType()->label(),
          'type_id'  => $g->getGroupType()->id(),
          'members'  => GroupsStatistics::countMembers($g),
          'contents' => GroupsStatistics::countContents($g),
          'by_type'  => GroupsStatistics::countContentsByType($g),
        ];
      }
    }

    return $groups_stats;
  }

  /**
   * Constructs the controller object.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The user from current session.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   */
  public function __construct(AccountInterface $current_user, EntityTypeManagerInterface $entityTypeManager, MessengerInterface $messenger, ModuleHandlerInterface $module_handler) {
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entityTypeManager;
    $this->messenger = $messenger;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('module_handler')
    );
  }

}
