<?php

namespace Drupal\yasm\Utility;

use Drupal\Core\Entity\ContentEntityType;
use Drupal\Core\Entity\EntityTypeInterface;

/**
 * Entities statistics class.
 */
class EntitiesStatistics {

  /**
   * Count entities.
   *
   * @param string $entity_id
   *   The entity id.
   * @param array $conditions
   *   Filter conditions with key => value. Empty array to not filter.
   *
   * @return int
   *   The entity count.
   */
  public static function count(string $entity_id, array $conditions = []) {
    $query = \Drupal::entityQuery($entity_id);

    if (!empty($conditions)) {
      foreach ($conditions as $key => $value) {
        if (substr($value, -1) === '%') {
          $query->condition($key, $value, 'LIKE');
        }
        else {
          $query->condition($key, $value);
        }
      }
    }

    return $query->count()->execute();
  }

  /**
   * Count entities with aggregation.
   */
  public static function aggregate(string $entity_id, array $aggregates = [], string $group_by = NULL, array $conditions = []) {
    $query = \Drupal::entityQueryAggregate($entity_id);

    if (!empty($group_by)) {
      $query->groupBy($group_by);
    }
    if (!empty($aggregates)) {
      foreach ($aggregates as $key => $value) {
        $query->aggregate($key, $value);
      }
    }
    if (!empty($conditions)) {
      foreach ($conditions as $key => $value) {
        if (substr($value, -1) === '%') {
          $query->condition($key, $value, 'LIKE');
        }
        else {
          $query->condition($key, $value);
        }
      }
    }

    $result = $query->execute();

    return $result;
  }

  /**
   * Get site content entities.
   */
  public static function getEntitiesInfo(array $conditions = []) {
    $entities = [];
    $definitions = \Drupal::entityTypeManager()->getDefinitions();
    if (!empty($definitions)) {
      foreach ($definitions as $definition) {
        if ($definition instanceof ContentEntityType) {
          // Collect entity info.
          $entities += self::getEntityAndBundlesInfo($definition, $conditions);
        }
      }
    }

    return $entities;
  }

  /**
   * Get entity and parent bundles info.
   */
  public static function getEntityAndBundlesInfo(EntityTypeInterface $entity, array $conditions = []) {
    // Entity data.
    $info = [];
    $entity_id = $entity->id();
    $info[$entity_id] = self::getEntityInfo($entity, $conditions);
    // Entity bundles.
    $entity_type = $entity->getBundleEntityType();
    if (!empty($entity_type)) {
      $bundles = \Drupal::entityTypeManager()->getStorage($entity_type)->loadMultiple();
      if (!empty($bundles)) {
        foreach ($bundles as $bundle) {
          // Collect bundle info.
          $info[$entity_id]['bundles'][] = self::getBundleInfo($entity, $bundle, $conditions);
        }
      }
    }

    return $info;
  }

  /**
   * Get entity basic data.
   */
  public static function getEntityInfo(EntityTypeInterface $entity, array $conditions = []) {
    return [
      'id'    => $entity->id(),
      'label' => $entity->getLabel(),
      'count' => self::countEntityElements($entity, $conditions),
    ];
  }

  /**
   * Get bundle basic data.
   */
  public static function getBundleInfo(EntityTypeInterface $entity, $bundle, array $conditions = []) {
    return [
      'id'    => $bundle->id(),
      'label' => $bundle->label(),
      'count' => self::countBundleElements($entity, $bundle, $conditions),
    ];
  }

  /**
   * Count entity elements.
   */
  public static function countEntityElements(EntityTypeInterface $entity, array $conditions = []) {
    return self::count($entity->id(), $conditions);
  }

  /**
   * Count bundle elements.
   */
  public static function countBundleElements(EntityTypeInterface $entity, $bundle, array $conditions = []) {
    $bundle_field = $entity->get('entity_keys');
    if (!empty($bundle_field['bundle'])) {
      $conditions += [$bundle_field['bundle'] => $bundle->id()];

      return self::count($entity->id(), $conditions);
    }

    return 0;
  }

}
